/* ------------------------------------------------------------------------------------------------ */
/* FILE :		Demo_3_MSP430.c																		*/
/*																									*/
/* CONTENTS :																						*/
/*				Demo for MSP430(X) boards															*/
/*				Operation of the RTOS is shown on the serial port									*/
/*				Tasks info & configuration through the serial port									*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/* INFO:																							*/
/*       How this demo works:																		*/
/*       6 Tasks are running, and also Adam & Eve (main) which is not displayed.					*/
/*       The RTOS timer tick interrupt callback handler goes through a state machine to control the	*/
/*       operations & triggers state changes of the 6 tasks.  The information is sent to the tasks	*/
/*       either by mailboxes or by events (see the defintion for USE_EVENTS).						*/
/*       Adam & Eve is periodically unblocked (semaphore) by the RTOS timertick callback.			*/
/*       Adam & Eve looks at the state opf all 6 tasks and send strings to print on stdout			*/
/*       through a mailbox.																			*/
/*       All 6 tasks monitor this mailbox and perform the printing on stdout.						*/
/*       Although a mutex should be used to provided exclusive access to stdout, interrupt			*/
/*       disabling & enabling is needed instead.  This is because if a mutex was used, the priority	*/
/*       inheritance would most likely kick in and change the priority of the printfing task.		*/
/*       That would create a weird display giving the impression tasks chenge priority for no		*/
/*       reasons.																					*/
/*																									*/
/*       For a full description of what is displayed, see:											*/
/*                    www.code-time.com/demo3.html													*/
/*																									*/
/* Copyright (c) 2012-2017, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.3 $																				*/
/*	$Date: 2017/04/27 15:48:27 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if (((OS_DEMO) != 3) && ((OS_DEMO) != -3))
	#error "Wrong OS_DEMO value" 
#endif

#include <msp430.h>
#include <ctype.h>
#include "Abassi.h"
#include "Platform.h"								/* Everything about the target platform is here	*/
#include "HWinfo.h"									/* Everything about the target board is here	*/
#if defined(__TI_COMPILER_VERSION__)
  #include <file.h>
#endif

/* ------------------------------------------------ */


void CLOCK_XT2_Init(void);
void SPI2_Init();
void UART0_Init(unsigned long baud);
void UART1_Init(unsigned long baud);
void UART2_Init(unsigned long baud);
void UART3_Init(unsigned long baud);
void UART4_Init(unsigned long baud);
void UART5_Init(unsigned long baud);

/* ------------------------------------------------------------------------------------------------ */
/* App definitions																					*/

#define TSK_REFRESH_RATE	5						/* Display refresh rate: 1/TSK_REFRESH_RATE sec	*/
#define RTC_REFRESH_RATE	1						/* RTC refresh rate related to TSK_REFRESH_RATE	*/

#ifndef USE_EVENTS
 #if ((OX_MAILBOX) == 0)
  #define USE_EVENTS		1						/* No mailbox, has to use events				*/
 #else
  #define USE_EVENTS		0						/* Mailboxes instead of events inform tasks		*/
 #endif
#endif
#define OP_ZERO_CNT			1						/* Operation requested by the ISR state machine	*/
#define OP_LOCK_MTX			2						/* These are the messages / events sent to the	*/
#define OP_UNLOCK_MTX		4						/* 6 tasks										*/

#define TSK_HIGH			0						/* Index of task info/descriptot in TskDsc[]	*/
#define TSK_RRA				1
#define TSK_RRB				2
#define TSK_RRC				3
#define TSK_STRVA			4
#define TSK_STRVB			5

#define OP_ROUND_ROBIN		0						/* Command line configuration operation			*/
#define OP_STARVE_RUN		1						/* These are what to modify in a task			*/
#define OP_STARVE_WAIT		2
#define OP_STARVE_PRIO		3
#define OP_PRIO				4

#ifndef MBX_DISP_SIZE
 #define MBX_DISP_SIZE		32						/* Depth of the mailbox for the display			*/
#endif

/* ------------------------------------------------------------------------------------------------ */
/* App variables																					*/
char Display[7][20] = { "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
                        "00.0s  Px      ",
						"00:00:00       "
                      };
#define LAST_DISP		6							/* Number of lines / tasks to display			*/
#define CURSOR_START()	printf("\033[15;0H");		/* Cursor movements								*/
#define CURSOR_NEXT(x)	printf("\033[15;%02dH", (x)+15);
#define CURSOR_BACK()	printf("\033[15;15H");
#define CURSOR_ADD()	printf("\033[15;16H");

typedef struct {									/* Hold the descriptions for the task monitored	*/
	TSK_t *Task;									/* Task descriptor								*/
  #if ((USE_EVENTS) == 0)
	MBX_t *Mailbox;									/* Mailbox when ISR state machine uses mailboxes*/
  #endif
	int    Running;									/* If this is the running task (ignoring A&E)	*/
	int    WasReset;								/* To inform task to reset running time counter	*/
	int    CPUcnt;									/* Number of timer ticks task has been running	*/
	int    WillBlk;									/* If the task is next to block on the mutex	*/
	int    WillMtx;									/* If the task is next to lock the mutex		*/
} TskDsc_t;

TskDsc_t TskDsc[LAST_DISP];

typedef struct {									/* Info sent to the task to update the display	*/
	char *Str;										/* String to print on the display				*/
	int  X;											/* X coordinate									*/
	int  Y;											/* Y coordinate									*/
} DispInfo_t;										/* It is sent to UpdateDisp()					*/

DispInfo_t DispInfo[MBX_DISP_SIZE];

volatile int g_AppRdy = 0;							/* To inform RTOS timer callback when the		*/
volatile int g_CmdIdx;								/* Index where to write next character from kyb	*/
volatile int g_DispIdx = 0;							/* Next entry in DispInfo[] to fill				*/
													/* application is fully configured				*/
TSK_t *AdamEve;										/* Adam & Eve task descriptor					*/
MTX_t *Mutex;										/* Mutex used by the task to show inheritance	*/
#if ((OS_MAILBOX) != 0)
  MBX_t *MbxDisp;									/* Mailbox to send display updates to the tasks	*/
#else												/* No mailboxes, use a circulare buffer			*/
  int g_DispRdIdx=0;								/* Index used by Fct to grab the display test	*/
#endif
SEM_t *SemaAE;										/* Semaphore to awake Adam & Eve				*/

#if ((OX_STARVE_RUN_MAX) != 0)
  extern TSK_t *g_StrvList;							/* Imported from Abassi.c						*/
#else
  TSK_t *g_StrvList = (TSK_t *)NULL;
#endif

#ifdef DEMO_XTRA_VARS								/* Platform specific variables					*/
  DEMO_XTRA_VARS
#endif

/* ------------------------------------------------------------------------------------------------ */
/* Apps functions																					*/

void   Fct(void);									/* A single function implements all the tasks	*/
int    GetKey(void);								/* Non-blocking keyboard input					*/
void   UpdateDisp(DispInfo_t *Disp);
#if defined(__TI_COMPILER_VERSION__)
  void CCSioInit(void);
#endif

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int main(void)
{
int   ccc;											/* Character read from the keyboard				*/
long  Clock;
int   Done;											/* If command decoding is done					*/
int   Error;										/* If error during command decoding				*/
int   ii;											/* General purpose								*/
int   IsrState;
int   jj;											/* General purpose								*/
int   kk;											/* General purpose								*/
int   LastTimTick;
long  LL;
int   Op;											/* Operation requested by the command			*/
int   RdIdx;										/* Read index in the command line string		*/
char *String;										/* Display sting being created					*/
int   TaskIdx;										/* Index of the task specified on command line	*/
int   Toggle;										/* Toggle use to control refresh of RTC			*/

static char Cmd[64];								/* Command line from keyboard					*/

/* ------------------------------------------------ */
/* 	Processor clock set-up							*/

	WDTCTL = WDTPW + WDTHOLD;						/* Turn off the watchdog timer					*/
													/* Done earlier in IAR with low_lelevl_init()	*/
	CLOCK_XT2_Init();

/* ------------------------------------------------ */
/* Set buffering here: tasks inherit main's stdios	*/
/* when Newlib reent is used & stdio are shared		*/ 

 #if !defined(__IAR_SYSTEMS_ICC__)
  {
  char SetV[3];										/* IAR needs non-NULL buf with _IONBF			*/
	setvbuf(stdin,  &SetV[0], _IONBF, 0);			/* By default, libraries flush the I/O			*/
	setvbuf(stdout, &SetV[1], _IONBF, 0);			/* buffer when full or when new-line			*/
	setvbuf(stderr, &SetV[2], _IONBF, 0);			/* Done before OSstart() to have all tasks with	*/
  }													/* the same stdio set-up						*/
 #endif

/* ------------------------------------------------ */
/* Start Abassi										*/

	OSstart();										/* Start Abassi								*/


	AdamEve = TSKmyID();							/* This demo needs to know when A&E is running	*/

/* ------------------------------------------------ */
/* SysTick Set-up									*/

	TIMERinit(((SYS_FREQ/100000L)*((long)OS_TIMER_US))/10L);
  #if defined(__TI_COMPILER_VERSION__)
	OSisrInstall(TIMER0_NAME, &TIMtick);
  #else
	OSisrInstall(TIMER0_NAME/2, &TIMtick);
  #endif


/* ------------------------------------------------ */
/* UART set-up										*/
/* 8 bits, No Parity, 1 Stop						*/

  #if ((UART_DEV) == 0)
	UART0_Init((long)(BAUDRATE));
  #elif ((UART_DEV) == 1)
	UART1_Init((long)(BAUDRATE));
  #elif ((UART_DEV) == 2)
	UART2_Init((long)(BAUDRATE));
  #elif ((UART_DEV) == 3)
	UART3_Init((long)(BAUDRATE));
  #elif ((UART_DEV) == 4)
	UART4_Init((long)(BAUDRATE));
  #elif ((UART_DEV) == 5)
	UART5_Init((long)(BAUDRATE));
  #endif

  #if defined(__TI_COMPILER_VERSION__)
 	CCSioInit();
  #endif


/* ------------------------------------------------ */
/* Application set-up								*/
													/* Need to create all services before allowing	*/
  #if ((OS_MAILBOX) != 0)							/* any task to run								*/
	MbxDisp                  = MBXopen("Disp", MBX_DISP_SIZE);
  #endif
	Mutex                    = MTXopen("Mutex");
	SemaAE                   = SEMopen("Sem A&E"); 

	TskDsc[TSK_HIGH].Task    = TSKcreate("HI   ", 2, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_RRA].Task     = TSKcreate("RRA  ", 4, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_RRB].Task     = TSKcreate("RRB  ", 4, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_RRC].Task     = TSKcreate("RRC  ", 4, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_STRVA].Task   = TSKcreate("STRVA", 7, TC_TSK_STACK, &Fct, 0);
	TskDsc[TSK_STRVB].Task   = TSKcreate("STRVB", 9, TC_TSK_STACK, &Fct, 0);

  #if ((OX_ROUND_ROBIN) < 0)
	TSKsetRR(TskDsc[TSK_HIGH].Task,  1*OS_TICK_PER_SEC);
	TSKsetRR(TskDsc[TSK_RRA].Task,   4*OS_TICK_PER_SEC);
	TSKsetRR(TskDsc[TSK_RRB].Task,   6*OS_TICK_PER_SEC);
	TSKsetRR(TskDsc[TSK_RRC].Task,   8*OS_TICK_PER_SEC);
	TSKsetRR(TskDsc[TSK_STRVA].Task, 1*OS_TICK_PER_SEC);
	TSKsetRR(TskDsc[TSK_STRVB].Task, 1*OS_TICK_PER_SEC);
  #endif

  #if ((OX_STARVE_RUN_MAX) < 0)
	TSKsetStrvRunMax(TskDsc[TSK_HIGH].Task,    2*OS_TICK_PER_SEC);
	TSKsetStrvRunMax(TskDsc[TSK_RRA].Task,     2*OS_TICK_PER_SEC);
	TSKsetStrvRunMax(TskDsc[TSK_RRB].Task,     2*OS_TICK_PER_SEC);
	TSKsetStrvRunMax(TskDsc[TSK_RRC].Task,     2*OS_TICK_PER_SEC);
	TSKsetStrvRunMax(TskDsc[TSK_STRVA].Task,   5*OS_TICK_PER_SEC);
	TSKsetStrvRunMax(TskDsc[TSK_STRVB].Task,   5*OS_TICK_PER_SEC);
  #endif

  #if ((OX_STARVE_WAIT_MAX) < 0)
	TSKsetStrvWaitMax(TskDsc[TSK_HIGH].Task,  (50*OS_TICK_PER_SEC)/10);
	TSKsetStrvWaitMax(TskDsc[TSK_RRA].Task,   (50*OS_TICK_PER_SEC)/10);
	TSKsetStrvWaitMax(TskDsc[TSK_RRB].Task,   (50*OS_TICK_PER_SEC)/10);
	TSKsetStrvWaitMax(TskDsc[TSK_RRC].Task,   (50*OS_TICK_PER_SEC)/10);
	TSKsetStrvWaitMax(TskDsc[TSK_STRVA].Task, (25*OS_TICK_PER_SEC)/10);
	TSKsetStrvWaitMax(TskDsc[TSK_STRVB].Task, (10*OS_TICK_PER_SEC)/10);
  #endif

  #if ((OX_STARVE_PRIO) < 0)
	TSKsetStrvPrio(TskDsc[TSK_HIGH].Task,  3);
	TSKsetStrvPrio(TskDsc[TSK_RRA].Task,   3);
	TSKsetStrvPrio(TskDsc[TSK_RRB].Task,   3);
	TSKsetStrvPrio(TskDsc[TSK_RRC].Task,   3);
	TSKsetStrvPrio(TskDsc[TSK_STRVA].Task, 3);
	TSKsetStrvPrio(TskDsc[TSK_STRVB].Task, 3);
  #endif

	for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {
		TSKsetArg(TskDsc[ii].Task, &TskDsc[ii]);
	  #if ((USE_EVENTS) == 0)
		TskDsc[ii].Mailbox  = MBXopen(NULL, 32);
	  #endif
		TskDsc[ii].CPUcnt   = 0;
		TskDsc[ii].Running  = 0;
		TskDsc[ii].WasReset = 0;
		TskDsc[ii].WillBlk  = 0;
		TskDsc[ii].WillMtx  = 0;
		TSKresume(TskDsc[ii].Task);
	}

/* ------------------------------------------------ */
/* Interrupts ON, the application runs				*/

	OSintOn();

/* ------------------------------------------------ */
/* Here we are										*/

	g_AppRdy = 1;									/* Inform the timer callback everything is OK	*/

	printf("\033[H\033[J");							/* Clear screen									*/
	puts("\n    Abassi Demo 3\n");

	puts("High  00.0s  P2     ");
	puts("RR-A  00.0s  P4     ");
	puts("RR-B  00.0s  P4     ");
	puts("RR-C  00.0s  P4     ");
	puts("ST-A  00.0s  P7     ");
	puts("ST-B  00.0s  P9     ");


	LastTimTick = G_OStimCnt;
	Clock       = 0L;

	g_CmdIdx = 0;
	Toggle   = 0;

	CURSOR_START();
	printf("Abassi Demo > ");						/* Prompt on Serial for the command line		*/

	for (;;) {

		while(0 != SEMwait(SemaAE, -1));			/* while only useful in eval version			*/

		if (++Toggle >= RTC_REFRESH_RATE) { 		/* RTC is not always refreshed					*/
			Toggle = 0;
			String = &Display[LAST_DISP][0];		/* Display line to update						*/

			jj = G_OStimCnt;
			kk = jj - LastTimTick;
			Clock += (long)kk;
			LastTimTick = jj;
															/* RTC display is up to 99:59:59 so		*/
			if (Clock >= (360000L*(long)OS_TICK_PER_SEC)) {	/* roll back 00:00:00 when max reached	*/
				Clock -= (360000L*(long)OS_TICK_PER_SEC);
			}

			LL = Clock
			   / (long)OS_TICK_PER_SEC;

			kk = (int)(LL/(3600L*10L));
			String[0] = '0'+kk;
			LL -= ((long)kk)*(3600L*10L);

			kk  = (int)(LL/(3600L));
			String[1] = '0'+kk;
			LL -= ((long)kk)*(3600L);

			kk  = (int)(LL/(600L));
			String[3] = '0'+kk;
			LL -= ((long)kk)*(600L);

			kk  = (int)(LL/(60L));
			String[4] = '0'+kk;
			LL -= ((long)kk)*(60L);

			kk  = (int)(LL/(10L));
			String[6] = '0'+kk;
			LL -= ((long)kk)*(10L);

			String[7] = '0'+(int)LL;

			DispInfo[g_DispIdx].Str = String;		/* Send the string to the running task			*/
			DispInfo[g_DispIdx].X   = (LAST_DISP)+5;
			DispInfo[g_DispIdx].Y   = 7;
		  #if ((OS_MAILBOX) != 0)
			MBXput(MbxDisp, (intptr_t)&DispInfo[g_DispIdx], 0);
		  #endif
			if (++g_DispIdx >= (sizeof(DispInfo)/sizeof(DispInfo[0]))) {
				g_DispIdx = 0;
			}
		}

		for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {	/* Update all tasks status		*/
			String = &Display[ii][0];				/* Display line to update						*/
			jj = (((long)TskDsc[ii].CPUcnt)*100L)/(OS_TICK_PER_SEC);
			if (jj > 9999) {						/* Can't display more than 99.9 (is /10)		*/
				jj = 9999;
			}

			kk = jj/1000;							/* Running time MS digit						*/
			String[0] = '0'+kk;
			jj = jj - (kk*1000);

			kk = jj/100;							/* Running time LS digit						*/
			String[1] = '0'+kk;
			jj = jj - (kk*100);

			kk = jj/10;								/* Running time fraction						*/
			String[3] = '0'+kk;

			if (TSKisSusp(TskDsc[ii].Task)) {		/* If the task is suspended, the priority		*/
				String[7] = 'S';					/* is replaced by "S "							*/
				String[8] = ' ';
			}
			else {									/* Regular priority field						*/
				String[7] = 'P';
				if (TskDsc[ii].Task->SuspRqst != 0) {
					String[8] = 'S';				/* When the task will be suspended, indicate so	*/
				}
				else {								/* Priority numerical value						*/
					String[8] = '0'+TSKgetPrio(TskDsc[ii].Task);
				}
			}

			if (TskDsc[ii].Running) {				/* If this is the running task, indicate so		*/
				String[11] = 'R';
			}
			else {									/* Not a running task, nothing in this field	*/
				String[11] = ' ';
			}

			if (TskDsc[ii].Task == g_StrvList) {	/* If the task is under starvation protection	*/
				String[12] = 'A';					/* Show the "A"									*/
			}
			else {									/* Not under starvation protection, nothing to	*/
				String[12] = ' ';					/* shown in this field							*/
			}

			if (TskDsc[ii].Task->MyMutex == Mutex) {
				String[13] = 'M';					/* If this task locks the mutex, show it		*/
			}
			else if (TskDsc[ii].WillMtx  != 0) {
				String[13] = 'm';					/* If the task will lock the mutex, show it		*/
			}
			else if (TskDsc[ii].Task->Blocker == Mutex) {
				String[13] = 'B';					/* If the task is blocked by the mutex, show it	*/
			}
			else if (TskDsc[ii].WillBlk  != 0) {
				String[13] = 'b';					/* if the task will block on the mutex, show it	*/
			}
			else {
				String[13] = ' ';					/* If the task not involved with the mutex, 	*/
			}										/* nothing to show in this field				*/

		 #if ((OS_MTX_INVERSION) != 0)
			if ((TSKgetPrio(TskDsc[ii].Task) != TskDsc[ii].Task->OriPrio)
		  #if ((OX_STARVE_RUN_MAX) != 0)
			&&  ((TskDsc[ii].Task != g_StrvList)
			  || (TskDsc[ii].Task->OriPrio != TskDsc[ii].Task->StrvPrio))
		  #endif
			&&  (TskDsc[ii].Task->MyMutex == Mutex)) {
				String[14] = 'I';					/* If the task has it's priority increased due	*/
			}										/* to priority inheritance, show this			*/
			else
		 #endif
			{
				String[14] = ' ';
			}
			DispInfo[g_DispIdx].Str = String;
			DispInfo[g_DispIdx].X   = 4+ii;
			DispInfo[g_DispIdx].Y   = 7;
													/* Send the string to display to running task	*/
		  #if ((OS_MAILBOX) != 0)
			MBXput(MbxDisp, (intptr_t)&DispInfo[g_DispIdx], 0);
		  #endif
			if (++g_DispIdx >= (sizeof(DispInfo)/sizeof(DispInfo[0]))) {
				g_DispIdx = 0;
			}
		}

		while (0 != (ccc = GetKey())) {				/* Check for a new key pressed on the keyboard 	*/
			IsrState = OSintOff();					/* When A&E is printing, don't clash with other	*/
			if (g_CmdIdx == 0) {					/* tasks										*/
				for (ii=0 ; ii<sizeof(Cmd) ; ii++) {
					putchar(' ');
				}
			}
			CURSOR_NEXT(g_CmdIdx);

			if (g_CmdIdx < (sizeof(Cmd)/sizeof(Cmd[0]))-2) {
				putchar(ccc);						/* A key was pressed , echo it					*/
			}										/* If command line buffer full, don't echo		*/

			if (ccc == '\r') {						/* Need to remap CR to '\n'						*/
				putchar('\n');						/* Kept in case uart driver filter not set		*/
			}
			else if (ccc == '\b') {					/* Backspace pressed, to erase previous key		*/
				if (g_CmdIdx != 0) {
					g_CmdIdx--;
					putchar(' ');
					putchar('\b');
				}
			}
			else {
				if (g_CmdIdx == 0) {				/* Add to the command line without overflow		*/
					CURSOR_ADD();
					for (ii=1 ; ii<sizeof(Cmd) ; ii++) {
						putchar(' ');
					}
					CURSOR_ADD();
				}
				if (g_CmdIdx < (sizeof(Cmd)/sizeof(Cmd[0]))-2) {
					Cmd[g_CmdIdx] = toupper(ccc);
					g_CmdIdx++;
				}
			}

			if ((ccc == '\r')						/* CR pressed, then process the command			*/
			||  (ccc == '\n')) {
				Cmd[g_CmdIdx] = '\0';
				TaskIdx = -1;
				RdIdx   = 0;
				while ((Cmd[RdIdx] == ' ')			/* Skip start of line white spaces				*/
				||     (Cmd[RdIdx] == ',')
				||     (Cmd[RdIdx] == '\t')) {
					RdIdx++;
				}

				Done  = 0;
				Error = 0;

				if (g_CmdIdx == RdIdx) {			/* When everything white space, nothing to do	*/
					Done = 1;
				}

				g_CmdIdx = 0;

				CURSOR_START();

				if ((0 == strncmp(&Cmd[RdIdx], "?", 1))
				||  (0 == strncmp(&Cmd[RdIdx], "INFO", 4))) {
					putchar('\n');
					putchar('\n');
					for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {
						printf("%s  -->  ",  TskDsc[ii].Task->TskName);
					  #if ((OX_ROUND_ROBIN) != 0)
						printf("RR %3d", TSKgetRR(TskDsc[ii].Task));
					  #endif
					  #if ((OX_STARVE_RUN_MAX) != 0)
						printf("  |  SP %1d", TSKgetStrvPrio(TskDsc[ii].Task));
						printf("  |  SR %3d", TSKgetStrvRunMax(TskDsc[ii].Task));
						printf("  |  SW %3d", TSKgetStrvWaitMax(TskDsc[ii].Task));
					  #endif
					  #if (((OS_MTX_INVERSION) != 0) || ((OS_STARVE_RUN_MAX) != 0))
						printf("  |  P %d",  TskDsc[ii].Task->OriPrio);
					  #endif
						putchar('\n');
					}
					putchar('\n');
					Done = 1;
					CURSOR_BACK();
				}

				if (0 == strncmp(&Cmd[RdIdx], "HI", 2)) {
					TaskIdx = TSK_HIGH;
					RdIdx += 2;
				}
				if (0 == strncmp(&Cmd[RdIdx], "RRA", 3)) {
					TaskIdx = TSK_RRA;
					RdIdx += 3;
				}
				if (0 == strncmp(&Cmd[RdIdx], "RRB", 3)) {
					TaskIdx = TSK_RRB;
					RdIdx += 3;
				}
				if (0 == strncmp(&Cmd[RdIdx], "RRC", 3)) {
					TaskIdx = TSK_RRC;
					RdIdx += 3;
				}
				if (0 == strncmp(&Cmd[RdIdx], "STRVA", 5)) {
					TaskIdx = TSK_STRVA;
					RdIdx += 5;
				}
				if (0 == strncmp(&Cmd[RdIdx], "STRVB", 5)) {
					TaskIdx = TSK_STRVB;
					RdIdx += 5;
				}
				if (Done == 0) {
					if (TaskIdx < 0) {
						CURSOR_BACK();				/* Set cursor after Abassi Demo >				*/
						ii = printf("Unrecognized task name");
						for( ; ii<sizeof(Cmd) ; ii++) {
							putchar(' ');
						}
						Error=1;
					}
				}

				Op = -1;							/* 2nd token if the service, check all			*/
				if ((Error == 0)
				&&  (Done == 0)) {
					while ((Cmd[RdIdx] == ' ')		/* Skip white spaces between 1st and 2nd token	*/
					||     (Cmd[RdIdx] == ',')
					||     (Cmd[RdIdx] == '\t')) {
						RdIdx++;
					}
					if (0 == strncmp(&Cmd[RdIdx], "RR", 2)) {
						Op = OP_ROUND_ROBIN;		/* Token names are re-mapped to operations #	*/
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "SR", 2)) {
						Op = OP_STARVE_RUN;
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "SP", 2)) {
						Op = OP_STARVE_PRIO;
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "SW", 2)) {
						Op = OP_STARVE_WAIT;
						RdIdx += 2;
					}
					if (0 == strncmp(&Cmd[RdIdx], "P", 1)) {
						Op = OP_PRIO;
						RdIdx += 1;
					}
					if (Op < 0) {
						Error = 1;
						CURSOR_BACK();				/* Set cursor after Abassi Demo >				*/
						ii = printf("Unrecognized property        ");
						for( ; ii<sizeof(Cmd) ; ii++) {
							putchar(' ');
						}
					}
				}

				if ((Error == 0)
				&&  (Done == 0)) {
					while ((Cmd[RdIdx] == ' ')		/* Skip white spaces between 2nd and 3rd token	*/
					||     (Cmd[RdIdx] == ',')
					||     (Cmd[RdIdx] == '\t')) {
						RdIdx++;
					}
					ii = 0;
					if (Cmd[RdIdx] == '\0') {
						Error = 1;
						CURSOR_BACK();				/* Set cursor after Abassi Demo >				*/
						ii = printf("Invalid number");
						for( ; ii<sizeof(Cmd) ; ii++) {
							putchar(' ');
						}
					}
					do {							/* 3rd token is the service numerical value		*/
						if ((Cmd[RdIdx] >= '0')
						&&  (Cmd[RdIdx] <= '9')) {
							ii = ii*10;
							ii += Cmd[RdIdx] - '0';
							RdIdx ++;
						}
						else if ((Cmd[RdIdx] == '\0')
						     ||  (Cmd[RdIdx] == '\r')
						     ||  (Cmd[RdIdx] == '\n')) {
							Done = 1;
						}
						else {
							Error = 1;
							CURSOR_BACK();			/* Set cursor after Abassi Demo >				*/
							ii = printf("Invalid number");
							for( ; ii<sizeof(Cmd) ; ii++) {
								putchar(' ');
							}
						}
					} while ((Error == 0)
					  &&     (Done  == 0));

					if (Error == 0) {				/* When everything OK, set the numerical value	*/
						switch(Op) {				/* to the service in the task					*/
						case OP_ROUND_ROBIN:
						  #if ((OX_ROUND_ROBIN) < 0)
							TSKsetRR(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						case OP_STARVE_PRIO:
						  #if ((OX_STARVE_PRIO) < 0)
							TSKsetStrvPrio(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						case OP_STARVE_RUN:
						  #if ((OX_STARVE_RUN_MAX) < 0)
							TSKsetStrvRunMax(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						case OP_STARVE_WAIT:
						  #if ((OX_STARVE_WAIT_MAX) < 0)
							TSKsetStrvWaitMax(TskDsc[TaskIdx].Task, ii);
						  #endif
							break;
						default:
							TSKsetPrio(TskDsc[TaskIdx].Task, ii);
							break;
						}
					}
				}
				if (Error == 0) {
					CURSOR_START();					/* Set cursor at beginning of line				*/
					printf("Abassi Demo > ");
					for (ii=0 ; ii<sizeof(Cmd) ; ii++) {
						putchar(' ');
					}
					CURSOR_BACK();
				}
				else {
					CURSOR_NEXT(g_CmdIdx);
				}
			}
			OSintBack(IsrState);
		}
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Tasks function																					*/
/*																									*/
/* All tasks use the same function.																	*/
/* The Task Argument is the custom task description data structure									*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void Fct(void)
{
DispInfo_t *Disp;									/* Information to send to UpdateDisp()			*/
int         ii;										/* General purpose								*/
intptr_t    Msg;									/* Message received from timer tick call back	*/
int         NoNewMsg;								/* If there are no new messages					*/
TskDsc_t   *MyInfo;									/* All the information about my task			*/

 #if !defined(__IAR_SYSTEMS_ICC__)
 #if ((OX_LIB_REENT_PROTECT) > 0)					/* In case reentrance protect is per task		*/
  char SetV[3];										/* IAR needs non-NULL buf with _IONBF			*/
	setvbuf(stdin,  &SetV[0], _IONBF, 0);			/* By default, libraries flush the I/O			*/
	setvbuf(stdout, &SetV[1], _IONBF, 0);			/* buffer when full or when new-line			*/
	setvbuf(stderr, &SetV[2], _IONBF, 0);
 #endif
 #endif

	MyInfo = (TskDsc_t *)TSKgetArg();				/* Get the argument for this task				*/

	for ( ; ; ) {
	  #if USE_EVENTS								/* Info from ISR state machine comes from 		*/
		NoNewMsg = EVTwait(0x0000, 0xFFFF, 0);		/* either an event flag or a mailbox			*/
		Msg      = EVTget();
	  #else
		NoNewMsg = MBXget(MyInfo->Mailbox, &Msg, 0);
	  #endif

		if (NoNewMsg == 0) {						/* When a valid sync trigger was received		*/
			if ((int)Msg & OP_ZERO_CNT) {			/* Request to zero the running time				*/
				MyInfo->CPUcnt   = 0;
				MyInfo->WasReset = 0;
			}
			if (((int)Msg & OP_LOCK_MTX)			/* Request from ISR to lock the mutex			*/
			&&  (MTXowner(Mutex) != TSKmyID())) {	/* Safety in case state machine has issues		*/
				MyInfo->WillBlk = 0;
				MyInfo->WillMtx = 0;
				MTXlock(Mutex, -1);
			}
			if (((int)Msg & OP_UNLOCK_MTX)			/* Request from ISR to unlock the mutex			*/
			&&  (MTXowner(Mutex) == TSKmyID())) {	/* Safety in case state machine has issues		*/
				MTXunlock(Mutex);
			}
		}
		ii = OSintOff();							/* Will print, don't want to be pre-empted		*/
	  #if ((OS_MAILBOX) != 0)
		if (0 ==  MBXget(MbxDisp, &Msg, 0)) {		/* Check if A&E has sent a string to display	*/
			Disp = (void *)Msg;
	  #else
		if (g_DispIdx != g_DispRdIdx) {
			Disp = &DispInfo[g_DispRdIdx];
			if (++g_DispRdIdx >= (sizeof(DispInfo)/sizeof(DispInfo[0]))) {
				g_DispRdIdx = 0;
			}
	  #endif
			UpdateDisp(Disp);
		}
		OSintBack(ii);
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* RTOS Timer call back function																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if USE_EVENTS
	#define SEND_INFO(_Idx, _Info)	EVTset(TskDsc[_Idx].Task, _Info);
#else
	#define SEND_INFO(_Idx, _Info)	MBXput(TskDsc[_Idx].Mailbox, _Info, 0)
#endif

void TIMcallBack(void)
{
int    AErunning;
int    ii;
int    jj;
int    RunIdx;
static int Cnt       =  0;
static int MtxCnt    =  0;
static int MtxOwner  =  5;
static int MtxSelect =  5;
static int MtxBlk    =  1;
static int MtxState  =  0;
static int SuspCnt   =  0;

	if (g_AppRdy == 0) {							/* If the configuration not done, skip			*/
		return;
	}

	Cnt++;											/* Check if A&E is running as we can't update	*/
	AErunning = (G_OStaskNow == AdamEve);			/* everything when it does						*/
													/* Update the CPU of the tasks					*/
	RunIdx = -1;									/* Invalidate running status of the  task		*/
	if (AErunning == 0) {							/* When A&E is running, don't update			*/
		for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {
			TskDsc[ii].Running = 0;
			if (G_OStaskNow == TskDsc[ii].Task) {
				TskDsc[ii].Running  = 1;
				RunIdx = ii;
			}
			if (TskDsc[ii].Task->Blocker != (SEM_t *)NULL) {
				if (TskDsc[ii].WasReset == 0) {
					SEND_INFO(ii, OP_ZERO_CNT);
					TskDsc[ii].WasReset = 1;
				}
			}
		 #if ((OX_ROUND_ROBIN) != 0)
		  #if ((OX_ROUND_ROBIN) < 0)
			if (TskDsc[ii].Task->RRmax != 0U)
		  #endif
			 {
				if (TskDsc[ii].Task->RRcnt <= 1U) {
					if (TskDsc[ii].WasReset == 0) {
						SEND_INFO(ii, OP_ZERO_CNT);
						TskDsc[ii].WasReset = 1;
					}
				}
			}
		  #endif
		}
	}

	for (ii=0 ; ii<sizeof(TskDsc)/sizeof(TskDsc[0]) ; ii++) {	/* Excluding A&E & Idles			*/
		if (TskDsc[ii].Running != 0) {				/* It is supposed to be running, check if so	*/
			TskDsc[ii].CPUcnt++;
		}
	}

	if (AErunning == 0) {
		switch(MtxState) {
		case 0:										/* State #0 : new mutex owner					*/
			if (++MtxSelect >= sizeof(TskDsc)/sizeof(TskDsc[0])) {
				MtxSelect = 0;
			}
			if (MtxSelect == 2) {					/* RRB if not the first locker of the mutex		*/
				MtxSelect = 4;
			}
		  #ifdef TINY_ABASSI						/* TINY_ABASI doesn't have staravation protect	*/
			if (MtxSelect >= 4 ) {					/* Skip ST-A & ST-B								*/
				MtxSelect = 0;
			}
		  #endif

			MtxOwner = MtxSelect;

			SEND_INFO(MtxOwner, OP_LOCK_MTX);		/* Tell new owner to lock the mutex				*/

			if (++MtxBlk >= 4) {					/* Select the task that will block				*/
				MtxBlk = 0;							/* Only Hi & RR can block on the mutex			*/
			}

			if (MtxBlk == MtxOwner) {				/* Get another task to block on the mutex		*/
				if (++MtxBlk >= 4) {
					MtxBlk = 0;
				}
			}
			TskDsc[MtxOwner].WillMtx = 1;			/* Display it will lock the mutex soon			*/
			TskDsc[MtxBlk].WillBlk = 1;				/* And this will be blocked 					*/
			MtxState++;
			break;
		case 1:										/* Waiting for the owner to lock the mutex		*/
			if (TskDsc[MtxOwner].Task->MyMutex == Mutex) {
				MtxState++;
			}
			break;
		case 2:										/* OK, the mutex owner is locking the mutex		*/
			SEND_INFO(MtxBlk, OP_LOCK_MTX);			/* Tell the task to be blocked to lock it		*/
			MtxState++;
			break;
		case 3:										/* Make sure the task gets blocked				*/
			if (TskDsc[MtxOwner].Task->Prio == TskDsc[MtxBlk].Task->Prio) {
				if (TSKisBlk(TskDsc[MtxBlk].Task)) {/* OK, the task is blocked on the mutex now		*/
					MtxState++;						/* This may take a while as it needs to run		*/
					MtxCnt = 0;
				}
			}
			else {									/* If to be suspended, must skip a few states	*/
				if (TskDsc[MtxOwner].Task->SuspRqst != 0) {	/* as != prio can't run at same time	*/
					MtxState = 4;
					MtxCnt   = 0;
					TskDsc[MtxOwner].WillMtx = 0;
					TskDsc[MtxBlk].WillBlk   = 0;
					TskDsc[MtxBlk].WillMtx   = 1;
				}
			}
			break;
		case 4:										/* State #4: Task is blocked on the mutex		*/
			if (MtxOwner == RunIdx) {				/* Make sure the owner is running long enough	*/
				jj = (MtxOwner == 0)				/* When HI task, is getting suspended			*/
				   ? (((15*OS_TICK_PER_SEC)+1) - TskDsc[0].CPUcnt)
				   :   ((5*OS_TICK_PER_SEC)+1);
				if (++MtxCnt > jj) {				/* Tell the owner to release the mutex			*/
					SEND_INFO(MtxOwner, OP_UNLOCK_MTX);
					MtxState++;
				}
			}
			break;
		case 5:										/* Wait for owner to unlock the mutex			*/
			if (TskDsc[MtxOwner].Task->MyMutex != Mutex) {
				MtxOwner = MtxBlk;					/* The new owner is the one that was blocked	*/
				MtxState++;
			}
			break;
		case 6:										/* Wait for new owner to lock the mutex			*/
			if (TskDsc[MtxOwner].Task->MyMutex == Mutex) {
				MtxState++;
				MtxCnt = 0;
			}
			break;
		case 7:
			if (MtxOwner == RunIdx) {				/* Wait a while before telling to release		*/
				jj = (MtxOwner == 0)				/* When HI task, is getting suspended			*/
				   ? (((15*OS_TICK_PER_SEC)+1) - TskDsc[0].CPUcnt)
				   :   ((5*OS_TICK_PER_SEC)+1);
				if (++MtxCnt > jj) {				/* Tell the owner to release the mutex			*/
					SEND_INFO(MtxOwner, OP_UNLOCK_MTX);
					MtxState++;
				}
			}
			break;
		default:									/* Wait for blocked to unlock					*/
			if (TskDsc[MtxOwner].Task->MyMutex != Mutex) {
				MtxState =  0;
				MtxOwner = -1;
			}
			break;
  		}

		if ((Cnt >= OS_TICK_PER_SEC/TSK_REFRESH_RATE)
		&&  (TSKisBlk(AdamEve))) {					/* Make sure A&E does not use all the CPU		*/
			SEMpost(SemaAE);						/* Tell A & E to refresh the display			*/
			Cnt = 0;
		}
	}

	SuspCnt++;
	if (SuspCnt == 10*OS_TICK_PER_SEC+3) {			/* Suspend / resume the high priority			*/
		TSKsuspend(TskDsc[TSK_HIGH].Task);
	}
	else if (SuspCnt > 30*OS_TICK_PER_SEC+3) {
		TSKresume(TskDsc[TSK_HIGH].Task);
		SuspCnt = 0;
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void UpdateDisp(DispInfo_t *Disp)
{
int jj;

	if ((Disp->X == 0)								/* Cursor at top of screen						*/
	&&  (Disp->Y == 0)) {
//		printf("\033[H");
		putchar(0x1B);
		putchar('[');
		putchar('H');
	}
	else {
//		printf("\033[%d;%dH%s", Disp->X, Disp->Y, Disp->Str); 
		putchar(0x1B);
		putchar('[');
		if (Disp->X < 10) {
			putchar('0'+Disp->X);
		}
		else {
			putchar('0'+(Disp->X/10));
			putchar('0'+(Disp->X%10));
		}
		putchar(';');
		if (Disp->Y < 10) {
			putchar('0'+Disp->Y);
		}
		else {
			putchar('0'+(Disp->Y/10));
			putchar('0'+(Disp->Y%10));
		}
		putchar('H');

		for (jj=0 ; Disp->Str[jj] != '\0' ; jj++){
			putchar(Disp->Str[jj]);
		}
													/* Set cursor at the next typing location		*/
//		printf("\033[%02d;%02dH", LAST_DISP+9, g_CmdIdx+15);
		putchar(0x1B);
		putchar('[');
		putchar('0'+(((LAST_DISP)+9)/10));
		putchar('0'+(((LAST_DISP)+9)%10));
		putchar(';');
		jj = g_CmdIdx+15;
		if (jj < 10) {
			putchar('0'+jj);
		}
		else {
			putchar('0'+(jj/10));
			putchar('0'+(jj%10));
		}
		putchar('H');
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Initialization functions																			*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void CLOCK_XT2_Init()
{
  #if ((OS_PLATFORM) == 0x00435438)
  int timeout;

	timeout = 5000;

	P11DIR  = 0x01
		    | 0x02
	        | 0x04;									/* set P11.0, P11.1 & P11.2 as output			*/
	P11SEL  = 0x01
		    | 0x02
		    | 0x04;									/* set P11.0/.1/.2 as ACLK/MCLK/SMCLK function	*/
	P5SEL   = P5SEL
		    | 0x04
		    | 0x08;									/* set P5.2 & P5.3 as XT2IN/XT2OUT				*/
	UCSCTL6 = 0x80									/* XT2 drive for 16 -> 24 MHz					*/
	        | 0x10;									/* XT2 source extyernally from the pin			*/
	UCSCTL3 = UCSCTL3 | 0x20;						/* set SELREF = 010 --> FLL clock src = REFOCLK	*/
	UCSCTL4	= 0x0005								/*	MCLK source == XT2CLK						*/
			| 0x0050								/* SMCLK source == XT2CLK						*/
			| 0x0500;								/*	ACLK source == XT2CLK						*/
	UCSCTL5 = 0x0001								/* MCLK : /2 									*/
	        | 0x0004								/* SMCLK: /16									*/
	        | 0x0100;								/* ACLK : /2									*/
	do {
		if (!(UCSCTL7 & 0x08)) break;
		UCSCTL7 = UCSCTL7 & ~(0x0F);				/* Clear XT2,XT1,DCO fault flags				*/
		SFRIFG1 = SFRIFG1 & ~(0x02);				/* Clear fault flags (SFRIFG1.OFIFG = 0)		*/
		timeout--;
	} while ((SFRIFG1 & 0x02) && (timeout != 0));	/* Test oscillator fault flag					*/

  #endif

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void TIMERinit(long Rate)
{
int PreSc;

	for (PreSc=0 ; PreSc<3 ; PreSc++) {				/* Find ther pre-scaling to use to have a count	*/
		if (Rate <= 65535L) {						/* held on 16 bits								*/
			break;
		}
		Rate = Rate >> 1;
	}

	if (Rate > 65535L) {
		Rate = 65535L;
	}
	else {
		Rate--;
	}

  #if ((OS_PLATFORM) == 0x00431611)
	TA0CTL	 = 0x0200								/* Timer A clock source is SMCLK				*/
			 | (PreSc << 6)							/* Divider selection							*/
			 | 0x0010;								/* Mode up count								*/
  #elif ((OS_PLATFORM) == 0x00435438)
	TA0CTL	 = 0x0100								/* Timer A clock source is ACLK					*/
			 | (PreSc << 6)							/* Divider selection							*/
			 | 0x0010;								/* Mode up count								*/
  #else
	#error "Invalid platform"
  #endif
	TA0CCR0	 = (int)Rate;
	TA0CCTL0 = CCIE;								/* Enable CCR0 interrupt of timer A0			*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void UART0_Init(unsigned long Baud)
{
  #if ((OS_PLATFORM) == 0x00431611)
	WDTCTL = WDTPW + WDTHOLD;						/* Stop watchdog timer							*/
	BCSCTL1 =  0x00;								/* XT2 turned on								*/
	BCSCTL2  = 0x88;								/* MCLK / SMCLK is XT2							*/
	do {
		int ii;
		IFG1 &= ~OFIFG;								/* Clear OSCFault flag							*/
		for (ii=0 ; ii<0xFF; ii++);					/* Time for flag to set							*/
	} while ((IFG1 & OFIFG) == OFIFG);				/* OSCFault flag still set?						*/

	UCTL0    = CHAR;								/* Start bit, 8 data bits, no parity, 1 stop	*/
	UTCTL0   = 0x20;								/* SMCLK is UART clock							*/
	U0BR0    = 0xFF &  (SYS_FREQ/BAUDRATE);			/* UART divider value							*/
	U0BR1    = 0xFF & ((SYS_FREQ/BAUDRATE) >> 8);
	UMCTL0   = 0x00;								/* modulation									*/
	ME1     |= UTXE0 | URXE0;						/* enable UART module							*/
	P3SEL   |= 0x30;								/* P3.4,5 = USART0 TXD/RXD						*/
	P3DIR   |= 0x10;								/* enable TXD0 as output						*/
	P3DIR   &= ~BIT5;								/* enable RXD0 as input							*/
  #endif
	return;
}

/* ------------------------------------------------------------------------------------------------ */

void UART2_Init(unsigned long baud)
{
  #if ((OS_PLATFORM) == 0x00435438)
  float n, x;

	P10SEL    = 0;
	P10DIR   |= (1<<7)								/* set P10.7 as output LED 1					*/
	         |  (1<<6);								/* set P10.6 as output LED 2					*/
	P9DIR    |= 1<<4;								/* set P9.4=1 (Transmitter) as output			*/
	P9DIR    &= ~(1<<5);							/* set P9.5=0 (Receiver) as input				*/
	P9SEL    |= (1<<4)								/* set P9.4=1 & P9.5=1 as TX/RX					*/
	         |  (1<<5);
	UCA2CTL1 |= 0x01;								/* UCSWRST = 1 (bit0 = 1)						*/
	UCA2CTL1 = (UCA2CTL1 | (1<<6) | 0x01)			/* select ACLK as BaudRate source clock			*/
	         & ~(1<<7);								/* (bit6 = 1; bit7 = 1) 						*/
	UCA2MCTL |= 0x01;								/* UCOS16 = 1									*/
	n        = SYS_FREQ / baud;						/* calculate UCBR								*/
	UCA2BR0  = (int)(n/16) % 256;
	UCA2BR1  = (int)(n/16) / 256;
	x        = (int)((n-(int)(n))*8+0.5);			/* calculate UCBRS								*/
	UCA2MCTL = UCA2MCTL | (int)(x)<<1;
	x        = (int)(((n/16 - (int)(n/16))*16) + 0.5);
	UCA2MCTL = UCA2MCTL | (int)(x)<<4;				/* calculate UCBRF								*/
													/* calculate registers for the BaudRate			*/
	UCA2CTL0 = 0x00;								/* Set Format Frame: 8 bit; no parity; 1 stop	*/
	UCA2CTL1 = UCA2CTL1 & ~(0x01);					/* UCSWRST = 0 (bit0 = 0)						*/
  #endif

	return;
}


/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if defined(__TI_COMPILER_VERSION__)

int UARTopen(const char *path, unsigned int flags, int fd)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTclose(int fd)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTread(int fd, char *buf, unsigned int count)
{

  #if ((OS_PLATFORM) == 0x00431611)
	while ((IFG1 & URXIFG0) == 0);
	*buf = RXBUF0;
  #elif ((OS_PLATFORM) == 0x00435438)
	while ((UCA2IFG & 0x01) == 0);
	*buf = UCA2RXBUF;
  #endif

	return(1);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTwrite(int fd, const char *buf, unsigned int count)
{
char ccc;
int  ii;

  #if ((OS_PLATFORM) == 0x00431611)
	for (ii=0 ; ii<count ; ii++) {
		ccc = buf[ii];
		if (ccc == '\n') {
			while ((IFG1 & UTXIFG0) == 0);
			TXBUF0 = '\r';
		}

		while ((IFG1 & UTXIFG0) == 0);
		TXBUF0 = ccc;
	}
  #elif ((OS_PLATFORM) == 0x00435438)
	for (ii=0 ; ii<count ; ii++) {
		ccc = buf[ii];

		if (ccc == '\n') {
			while((UCA2IFG & 0x02) == 0);
			UCA2TXBUF = '\r';
		}
		while (!(UCA2IFG & 0x02));
		UCA2TXBUF = ccc;
	}
  #endif

	return(count);
}

/* ------------------------------------------------------------------------------------------------ */

off_t UARTseek(int fd, off_t off, int start)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTunlink(const char *path)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTrename(const char *old, const char *new)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

void CCSioInit(void)
{

	add_device("uart", _MSA, UARTopen, UARTclose, UARTread, UARTwrite, UARTseek, UARTunlink, UARTrename);
	freopen("uart:stdout", "w", stdout);
	setvbuf(stdout, NULL, _IONBF, 0);
	freopen("uart:stdin", "r", stdin);
	setvbuf(stdin, NULL, _IONBF, 0);

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

#elif defined(__ICC430__)

/* ------------------------------------------------------------------------------------------------ */
size_t __write(int dsc, unsigned char const *sss, size_t Size)
{
char ccc;
int	 ii;

  #if ((OS_PLATFORM) == 0x00431611)
	for (ii=0 ; ii<Size ; ii++) {
		ccc = sss[ii];
		if (ccc == '\n') {
			while ((IFG1 & UTXIFG0) == 0);
			TXBUF0 = '\r';
		}

		while ((IFG1 & UTXIFG0) == 0);
		TXBUF0 = ccc;
	}
  #elif ((OS_PLATFORM) == 0x00435438)
	for (ii=0 ; ii<Size ; ii++) {
		ccc = sss[ii];
		if (ccc == '\n') {
			while((UCA2IFG & 0x02) == 0);
			UCA2TXBUF = '\r';
		}
		while (!(UCA2IFG & 0x02));
		UCA2TXBUF = ccc;
	}
  #endif

	return(ii);
}

#endif

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
int ccc;

	ccc = 0;

  #if ((OS_PLATFORM) == 0x00431611)
	if ((IFG1 & URXIFG0) != 0) {
		ccc = RXBUF0;
	}
  #elif ((OS_PLATFORM) == 0x00435438)
	if ((UCA2IFG & 0x01) != 0) {
		ccc = UCA2RXBUF;
	}
  #endif

	return(ccc);
}

/* EOF */
